$(function(){
    // Book Model
    var Book = Backbone.Model.extend({
        defaults: function() {
            return {
                title: "enter title",
                author: "enter author"
            };
        },
        
        clear: function() {
            this.destroy();
        }

    });
    
    // BooksCollection
    var BooksCollection = Backbone.Collection.extend({
        model: Book,
        url: "/books.json"
    });
    
    // instantiate BooksCollection
    var Library = new BooksCollection;

	
	var BookView = Backbone.View.extend({
		// we will create new element, so use tagName
		tagName: "li",
		
		events: {
			"click a[item-action=delete]": "clear",
			"click a[item-action=edit]": "edit",
			"click a#submit": "onSubmit"			
		},

		// our template for book view
		template: _.template($("#book-template").html()),
		template_form: _.template($("#book-form").html()),
		
		initialize: function() {
			// something to do on startup?
		}, 
		
		render: function() {
			this.$el.html( this.template({ model: this.model }) );
			return this;
		},
		
		clear: function() {
            this.$el.remove();
            this.model.clear();
        },
		
		edit: function() {	
			// add a class to the element to enable easy styling with css
            this.$el.addClass("edit");
			// render form template
            this.$el.html(this.template_form({model:this.model}));        
		},
		
		onSubmit: function() {
			var val_title = this.$("input[name=title]").val();
            var val_author = this.$("input[name=author]").val();
            this.model.set({
                title: val_title,
                author: val_author
            });

            if (! this.model.hasChanged() )
                this.render();
		}
		
	});
	
	
	var AppView = Backbone.View.extend({
	  	el: $("#app"),
	  	template: _.template($("#library-template").html()),
		template_form: _.template($("#book-form").html()),

		events: {
			"click a#add-book": "onAddBook",
			"click #new-book a#submit": "onAddBookSubmit"
		},
		
	  	initialize: function() {				
			// bind addOne, addAll and render functions to this object
			_.bindAll(this, 'render', 'addOne', 'addAll');
				
			this.render();     

			Library.bind('add', this.addOne, this);
			Library.bind('reset', this.addAll, this);			
							
			Library.fetch();
    	},
		
		addAll: function(collection) {
			collection.each(this.addOne);
		},
		
		addOne: function(book) {
			var view = new BookView({model: book});
			//console.log("addOne, book", book, "view", view);
			this.$("ul#books-list").append( view.render().el );
		},
		
		onAddBook: function() {
			var container = this.$("#new-book") || $('<div id="new-book"></div>');			
			container.html( this.template_form({ model: new Book }) );
			this.$el.append(container);
		},
		
		onAddBookSubmit: function() {
			var val_author = this.$("#new-book input[name=author]").val();
            var val_title = this.$("#new-book input[name=title]").val();
            Library.create({
                title: val_title,
                author: val_author
            });
            this.$("#new-book").empty();
		},

    	render: function() {
			this.$el.html( this.template() );
		}
	});

	var App = new AppView;
	
});
